<?php
/**
 * Created by JetBrains PhpStorm.
 * User: paulopimenta
 * Date: 20/02/13
 * Time: 13:03
 * To change this template use File | Settings | File Templates.
 */

namespace Planejamento\Logic\Cardapio;

use Estoque\Util\Parametros;
use Geral\Entities\Servico;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Planejamento\Entities\Logintcard;
use \Planejamento\Entities\Cardadia;
use Planejamento\Logic\Cardapio\Exception\RestricoesCardadia;
use Planejamento\Logic\Cardapio\Exception\SubstituiPratoException;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use Geral\Entities\Lista;

class SubstituiPrato
{
    const TROCA_PRATO = 'TROCA_PRATO_CARD';

    public static function verificaMes($datainicial, $datafinal)
    {
        if (empty($datainicial) || empty($datafinal)) {
            RestricoesCardadia::informeMes();
        }

        $mesDataInicial = \Geral\Util\DateUtil::getMes($datainicial);
        $mesDataFinal   = \Geral\Util\DateUtil::getMes($datafinal);

        if ($mesDataInicial != $mesDataFinal) {
            RestricoesCardadia::mesmoMesDtInicialDtFinal();
        }
    }

    public static function trocaPrato(
        $nrorg, $filial, $servico, $dtinicial, $dtfinal, $pratoatual,
        $filiauxpratoatual, $pratonovo, $filiauxpratonovo, $cdoperador
    )
    {
        $listaFili = '_';
        $listaServ = '_';
        foreach ($filial as $fili) {
            $listaFili .= $fili . '_';
            foreach ($servico as $serv) {
                if ($fili == $serv['CDFILIAL']) {
                    $listaServ .= $serv['CDSERVFILI'] . '_';
                }
            }
        }

        $prato = Cardadia::getRepository()->verificaPratoCardapio(
            $nrorg, $listaFili, $listaServ, $dtinicial, $dtfinal,
            $pratoatual, $filiauxpratoatual
        );

        if ($prato) {
            $nvproduto
                = \Geral\Util\PictureProduto::retornaNumeroMaximoDeNiveis();

            $stNrFili = \Geral\Util\Novocodigo::geraCodigo(
                \Geral\Util\Ambiente::getNrorg(), 'LISTA'
            );
            $stNrServ = \Geral\Util\Novocodigo::geraCodigo(
                \Geral\Util\Ambiente::getNrorg(), 'LISTA'
            );

            foreach ($filial as $fili) {
                Lista::insere($stNrFili, $fili, $nrorg);
                foreach ($servico as $serv) {
                    if ($fili == $serv['CDFILIAL']) {
                        Lista::insere(
                            $stNrServ,
                            $fili . $serv['CDSERVFILI'],
                            $nrorg
                        );
                    }
                }
            }

            $stLogErro = self::trocaPratoCard(
                $stNrFili, $stNrServ, $dtinicial, $dtfinal, $pratoatual,
                $filiauxpratoatual, $pratonovo, $filiauxpratonovo, 'F',
                $nvproduto, $cdoperador
            );

            Lista::deleta($nrorg, $stNrFili);
            Lista::deleta($nrorg, $stNrServ);

            $errosLog = self::tratamentoLogErro(
                $stLogErro['P_NRLOGINTCARD'], $nrorg
            );

            return $errosLog;
        } else {
            RestricoesCardadia::pratoNaoExisteCardapioInformado();
        }
    }


    public static function tratamentoLogErro($stLogErro, $nrorg)
    {
        if (($stLogErro)) {
            $logErros = Logintcard::getObjectsByWhere(
                'NRLOGINTCARD = ? AND NRORG = ?', array($stLogErro, $nrorg)
            );

            $mensagem = array();

            /** @var $erro Logintcard */
            foreach ($logErros as $erro) {
                $params = array(
                    $erro->getDtcardestlog(),
                    $erro->getCdfildestlog(),
                    $erro->getCdpratlog(),
                    $erro->getCdserdestlog(),
                );

                $numErro = $erro->getIdnumerro();
                switch ($numErro) {
                case Logintcard::ERRO_ZERO:
                    $mensagem[] = tk_get_message_by_name(
                        Logintcard::MSG_ERRO_0, $params
                    );
                    break;
                case Logintcard::ERRO_UM:
                    $mensagem[] = tk_get_message_by_name(
                        Logintcard::MSG_ERRO_1, $params
                    );
                    break;
                case Logintcard::ERRO_DOIS:
                    $mensagem[] = tk_get_message_by_name(
                        Logintcard::MSG_ERRO_2, $params
                    );
                    break;
                case Logintcard::ERRO_TRES:
                    $mensagem[] = tk_get_message_by_name(
                        Logintcard::MSG_ERRO_3, $params
                    );
                    break;
                }
            }

            return $mensagem;
        }

        return true;
    }


    public static function trocaPratoCard(
        $stNrFili, $stNrServ, $dtinicial, $dtfinal, $pratoatual, $filiauxpratoatual,
        $pratonovo, $filiauxpratonovo, $modoproc, $nvproduto, $cdoperador
    )
    {

        $procedure = new Procedure(self::TROCA_PRATO);
        $procedure->addParam(
            new Param (
                'P_LISTAFILI',
                Param::PARAM_INPUT,
                $stNrFili,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_LISTASERV',
                Param::PARAM_INPUT,
                $stNrServ,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_DTINICIAL',
                Param::PARAM_INPUT,
                $dtinicial,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_DTFINAL',
                Param::PARAM_INPUT,
                $dtfinal,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_CDPRATORIG',
                Param::PARAM_INPUT,
                $pratoatual,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_CDFAPRATORIG',
                Param::PARAM_INPUT,
                $filiauxpratoatual,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_CDPRATDEST',
                Param::PARAM_INPUT,
                $pratonovo,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_CDFAPRATDEST',
                Param::PARAM_INPUT,
                $filiauxpratonovo,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_MODOPROC',
                Param::PARAM_INPUT,
                $modoproc,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_CDNVPRODUTO',
                Param::PARAM_INPUT,
                $nvproduto,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_CDOPERADOR',
                Param::PARAM_INPUT,
                $cdoperador,
                Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param (
                'P_NRLOGINTCARD',
                Param::PARAM_OUTPUT,
                null,
                Param::PARAM_TYPE_STR,
                20
            )
        );
        return $procedure->call();
    }

    public static function montaStringServicos($servicos, $grupoServico = null)
    {
        $strServico = '';

        if (empty($grupoServico) === true) {
            foreach ($servicos as $servico) {
                $strServico .= '_' . $servico['CDSERVFILI'];
            }
        } else {
            /** @var $objsServico Servico[] */
            $objsServico = Servico::getObjectsByWhere(
                'CDGRUPSERV = ? AND CDFILIAL = ? AND NRORG = ?',
                array($grupoServico, Ambiente::getCdFilial(), Ambiente::getNrorg())
            );

            foreach ($objsServico as $servico) {
                $strServico .= '_' . $servico->getCdservfili();
            }
        }

        return $strServico;
    }


    public static function validaSubstituicao($dtinicial, $dtfinal)
    {
        try {
            /*$utilDtProc = Parametros::utilDataProcessamento();
            if ($utilDtProc === 'N') {
                RestricoesCardadia::impossivelTrocarPrato();
            }*/

            SubstituiPrato::verificaMes($dtinicial, $dtfinal);

            $dtProc         = Parametros::getDataProcessamento();
            $dtFinalAntProc = DateUtil::comparaDatas(
                DateUtil::getDataDeString($dtinicial),
                DateUtil::getDataDeString($dtProc)
            );
            if ($dtFinalAntProc) {
                $mensagem = tk_get_message_by_name('pla06600_periodoAntDtProc');
                tk_message($mensagem, 'error');
//                $sim = array(
//                    "label"      => "Sim",
//                    "method"     => array(
//                        "\\Windows\\Pla06600\\Pla06600Cardapio\\Facade",
//                        "realizaTrocaPrato"
//                    ),
//                    'params_arr' => array(
//                        $nrorg, $filial, $cdservfili, $dtinicial, $dtfinal,
//                        $pratoatual, $filiauxpratoatual, $pratonovo,
//                        $filiauxpratonovo, $cdoperador
//                    ),
//                );
//
//                $nao = array(
//                    "label" => "Não",
//                );
//
//                tk_confirm($mensagem, $sim, $nao);
                return false;
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

}
